/*
 * Copyright (C) 2025 iopsys Software Solutions AB
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2.1
 * as published by the Free Software Foundation
 *
 *	  Author: Amin Ben Romdhane <amin.benromdhane@iopsys.eu>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libbbfdm-ubus/bbfdm-ubus.h>

#ifndef WIFIDMD_DISABLE_LEGACY_WIFI
#include "wifi.h"
#endif

#ifdef WIFIDMD_ENABLE_WIFI_DATAELEMENTS
#include "dataelements.h"
#endif

#define DEFAULT_LOG_LEVEL LOG_INFO

/* ********** DynamicObj ********** */
static DM_MAP_OBJ tDynamicObj[] = {
/* parentobj, nextobject, parameter */

#ifndef WIFIDMD_DISABLE_LEGACY_WIFI
{"Device.", tDeviceWiFiObj, NULL},
#endif

#ifdef WIFIDMD_ENABLE_WIFI_DATAELEMENTS
{"Device.WiFi.", tDeviceWiFiDataElementsObj, NULL},
#endif

{0}
};

static void usage(char *prog)
{
	fprintf(stderr, "Usage: %s [options]\n", prog);
	fprintf(stderr, "\n");
	fprintf(stderr, "options:\n");
	fprintf(stderr, "    -l  <0-7> Set the loglevel\n");
	fprintf(stderr, "    -h  Displays this help\n");
	fprintf(stderr, "\n");
}

#ifdef WIFIDMD_ENABLE_WIFI_DATAELEMENTS
static void wifi_dataelements_refresh(struct uloop_timeout *timeout);

static struct uloop_timeout wifi_dataelements_timer = {
	.cb = wifi_dataelements_refresh
};

static void wifi_dataelements_refresh(struct uloop_timeout *timeout)
{
	BBF_INFO("Refreshing WiFi DataElements Dump Output due to 'wifi.dataelements.refresh', 'wifi.dataelements.Associated', or 'wifi.dataelements.Disassociated' events");
	init_wifi_dataelements_dump();
}

static void wifi_data_elements_cb(struct ubus_context *ctx, struct ubus_event_handler *ev,
			  const char *type, struct blob_attr *msg)
{
	BBF_INFO("Received WiFi DataElements event: '%s'", type);

	// Cancel the previous scheduled refresh
	uloop_timeout_cancel(&wifi_dataelements_timer);

	// Schedule a refresh after a 2-second delay
	uloop_timeout_set(&wifi_dataelements_timer, 2000);
}
#endif

int main(int argc, char **argv)
{
	struct bbfdm_context bbfdm_ctx = {0};
	int log_level = DEFAULT_LOG_LEVEL;
	int c = 0;

	while ((c = getopt(argc, argv, "hl:")) != -1) {
		switch (c) {
		case 'l':
			log_level = (int)strtod(optarg, NULL);
			if (log_level < 0 || log_level > 7) {
				log_level = DEFAULT_LOG_LEVEL;
			}
			break;
		case 'h':
			usage(argv[0]);
			return EXIT_SUCCESS;
		default:
			usage(argv[0]);
			return EXIT_FAILURE;
		}
	}

	memset(&bbfdm_ctx, 0, sizeof(struct bbfdm_context));

	bbfdm_ubus_set_service_name(&bbfdm_ctx, SERVICE_NAME);
	bbfdm_ubus_set_log_level(log_level);
	bbfdm_ubus_load_data_model(tDynamicObj);

	openlog(SERVICE_NAME, LOG_CONS | LOG_PID | LOG_NDELAY, LOG_LOCAL1);

	if (bbfdm_ubus_regiter_init(&bbfdm_ctx))
		goto out;

#ifdef WIFIDMD_ENABLE_WIFI_DATAELEMENTS
	struct ubus_event_handler ev_wifi_data_elements = {
		.cb = wifi_data_elements_cb,
	};

	init_wifi_dataelements_dump();

	if (ubus_register_event_handler(&bbfdm_ctx.ubus_ctx, &ev_wifi_data_elements, "wifi.dataelements.*"))
		goto out;
#endif

	uloop_run();

out:

#ifdef WIFIDMD_ENABLE_WIFI_DATAELEMENTS
	clean_wifi_dataelements_dump();

	ubus_unregister_event_handler(&bbfdm_ctx.ubus_ctx, &ev_wifi_data_elements);
#endif

	bbfdm_ubus_regiter_free(&bbfdm_ctx);

	closelog();

	return 0;
}
